import { Country } from '@/classes/Enum/Country'

export class IbanGenerator {
  generate (country: Country) {
    const t = this.getBankAccount(country)
    const n = this.getNationalCheckDigit(t)
    const r = this.getBankCode(country)
    switch (country) {
      case Country.ESTONIA:
        return country.toUpperCase() + this.getIbanCheckDigits(r + t + n + '141400') + r + t + n
      case Country.LITHUANIA:
        return country.toUpperCase() + this.getIbanCheckDigits(r + t + '212900') + r + t
      default:
        return country.toUpperCase() + this.getIbanCheckDigits(r + t + '252100') + r + t
    }
  }

  getBankAccount (country: Country) {
    let t
    const i = BigInt(97)
    const o = BigInt((t = Country.POLAND === country
      ? Math.floor(9999999 * Math.random()).toString().padStart(7, '0') +
        Math.floor(9999999 * Math.random()).toString().padStart(7, '0')
      : Math.floor(99999 * Math.random()).toString().padStart(5, '0') +
        Math.floor(9999 * Math.random()).toString().padStart(4, '0')) + '00')
    return t + String(i - (o % i)).padStart(2, '0')
  }

  getNationalCheckDigit (e: string) {
    const t = [7, 1, 3, 7, 1, 3, 7, 1, 3, 7, 1, 3, 7, 1, 3, 7, 1, 3, 7]
    const n = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
    let r = 0
    const i = e.length
    let o = 18
    for (let a = i - 1; a > -1; a--) {
      n[o] = t[o] * +e[a]
      o--
    }
    o = 18
    for (let a = i - 1; a > -1; a--) {
      r -= -n[o]
      o--
    }

    return String(r % 10 !== 0 ? 10 - r % 10 : 0)
  }

  getBankCode (e: Country) {
    const t = ['16000003', '11602231', '10301612', '18900002', '15002240', '19401018', '19101152']
    const n = ['2200', '1010', '3300', '7700', '1700', '4204']
    const r = ['20900', '72900', '74000', '40100', '21200', '72300', '21700', '21400', '70440', '71800', '73000']
    switch (e) {
      case Country.POLAND:
        return t[Math.floor(Math.random() * t.length)]
      case Country.ESTONIA:
        return n[Math.floor(Math.random() * n.length)]
      default:
        return r[Math.floor(Math.random() * r.length)]
    }
  }

  getIbanCheckDigits (e: string) {
    return String(BigInt('98') - (BigInt(e) % BigInt('97'))).padStart(2, '0')
  }
}

export default new IbanGenerator()
